import {Context, DefaultState, DefaultContext, Middleware, Next} from 'koa';
import Router from 'koa-router';
import {createHash} from 'crypto';
import {promises} from 'fs'
import {extname} from 'path';
/**
 * content file or directory referrer
 */
export interface ContentFile{
    getHash():Promise<string>;
    getDesc():Promise<object|null>;
    readonly path: string;
    readonly type: string;
}
type ContentFileConstructor =  (new (path:string,desc?:object) => ContentFile)&{content_type:string};
export const createDefaultClass = (type:string):ContentFileConstructor=>{
    let cons = class implements ContentFile{
        readonly path: string;
        type = type;
        static content_type = type;

        constructor(path:string,option?:object){
            this.path = path;
        }
        async getDesc(): Promise<object|null> {
            return null;
        }
        async getHash():Promise<string>{
            const stat = await promises.stat(this.path);
            const hash = createHash("sha512");
            hash.update(extname(this.type));
            hash.update(stat.mode.toString());
            //if(this.desc !== undefined)
            //    hash.update(JSON.stringify(this.desc));
            hash.update(stat.size.toString());
            return hash.digest("base64");
        }
    };
    return cons;
}
let ContstructorTable:{[k:string]:ContentFileConstructor} = {};
export function registerContentReferrer(s: ContentFileConstructor){
    console.log(`registered content type: ${s.content_type}`)
    ContstructorTable[s.content_type] = s;
}
export function createContentFile(type:string,path:string,option?:object){
    const constructorMethod = ContstructorTable[type];
    if(constructorMethod === undefined){
        console.log(type);
        throw new Error("undefined");
    }
    return new constructorMethod(path,option);
}
export function getContentFileConstructor(type:string): ContentFileConstructor|undefined{
    const ret = ContstructorTable[type];
    return ret;
}